/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package j3d.examples.ballGrid;

// WrapCheckers3D.java
// Andrew Davison, January 2003, dandrew@ratree.psu.ac.th

/* Illustrates how to create a simple world. The checkboard
   floor is created by CheckerFloor. The background, lighting,
   key controls, and initial user positioning is done here.

   Most of this will stay the same from one example to another.

   floatingSphere() shows how to create a coloured, shiny object
   which is affected by the lighting in the world.

   The scene graph display code is in WrapCheckers3D() and 
   createSceneGraph().
*/


import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;
import j3d.examples.checkers.CheckerFloor;

import javax.media.j3d.*;
import javax.swing.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import java.awt.*;

// import com.tornadolabs.j3dtree.*;    // for displaying the scene graph


public class CheckerPanelCarl extends JPanel {
    private static final int PWIDTH = 512;   // size of panel
    private static final int PHEIGHT = 512;

    private static final int BOUNDSIZE = 100;  // larger than world

    private static final Point3d USERPOSN = new Point3d(0, 5, 20);
    // initial user position

    private SimpleUniverse su;
    private BranchGroup sceneBG;
    private BoundingSphere bounds;

    public CheckerPanelCarl() {
        setLayout(new BorderLayout());
        setOpaque(false);
        setPreferredSize(new Dimension(PWIDTH, PHEIGHT));

        GraphicsConfiguration config =
                SimpleUniverse.getPreferredConfiguration();
        Canvas3D canvas3D = new Canvas3D(config);
        add(BorderLayout.CENTER, canvas3D);
        canvas3D.setFocusable(true);
        canvas3D.requestFocus();

        su = new SimpleUniverse(canvas3D);


        createSceneGraph();
        initUserPosition();
        // set user's viewpoint
        orbitControls(canvas3D);
        // controls for moving the viewpoint

        su.addBranchGraph(sceneBG);

        // j3dTree.updateNodes( su );    // build the tree display window

    } // end of WrapCheckers3D()


    private void createSceneGraph() {
        sceneBG = new BranchGroup();
        bounds = new BoundingSphere(new Point3d(0, 0, 0), BOUNDSIZE);

        lightScene();         // add the lights
        addBackground();      // add the sky
        sceneBG.addChild(new CheckerFloor().getBG());  // add the floor

        populateScene(30, 10, 30, .05f);     // add the floating spheres
      //populateScene(50, 20, 1, .05f);     // add the floating spheres
      // populateCylinders(10, 1, 50, .05f);
     //   populateEyeFields(12, 12, 1f, 1f);
        sceneBG.compile();   // fix the scene
    } // end of createSceneGraph()


    private void lightScene()
            /* One ambient light, 2 directional lights */ {
        Color3f white = new Color3f(1.0f, 1.0f, 1.0f);

        // Set up the ambient light
        AmbientLight ambientLightNode = new AmbientLight(white);
        ambientLightNode.setInfluencingBounds(bounds);
        sceneBG.addChild(ambientLightNode);

        // Set up the directional lights
        Vector3f light1Direction = new Vector3f(-1.0f, -1.0f, -1.0f);
        // left, down, backwards
        Vector3f light2Direction = new Vector3f(1.0f, -1.0f, 1.0f);
        // right, down, forwards

        DirectionalLight light1 =
                new DirectionalLight(white, light1Direction);
        light1.setInfluencingBounds(bounds);
        sceneBG.addChild(light1);

        DirectionalLight light2 =
                new DirectionalLight(white, light2Direction);
        light2.setInfluencingBounds(bounds);
        sceneBG.addChild(light2);
    }  // end of lightScene()


    private void addBackground() {
        Background back = new Background();
        back.setApplicationBounds(bounds);
        back.setColor(0.17f, 0.65f, 0.92f);    // sky colour
        sceneBG.addChild(back);
    }  // end of addBackground()


    private void orbitControls(Canvas3D c) {
        OrbitBehavior orbit =
                new OrbitBehavior(c, OrbitBehavior.REVERSE_ALL);
        orbit.setSchedulingBounds(bounds);

        ViewingPlatform vp = su.getViewingPlatform();
        vp.setViewPlatformBehavior(orbit);
    }


    private void initUserPosition()
            // Set the user's initial viewpoint using lookAt()
    {
        ViewingPlatform vp = su.getViewingPlatform();
        TransformGroup steerTG = vp.getViewPlatformTransform();

        Transform3D t3d = new Transform3D();
        steerTG.getTransform(t3d);

        // args are: viewer posn, where looking, up direction
        t3d.lookAt(USERPOSN, new Point3d(0, 0, 0), new Vector3d(0, 1, 0));
        t3d.invert();

        steerTG.setTransform(t3d);
    }  // end of initUserPosition()


    // ---------------------- floating sphere -----------------


    private void populateScene(int i, int j, int k, float r) {
        Appearance blueApp = getAppearance();

        // position the sphere
        Transform3D t3d;
        t3d = new Transform3D();
        TransformGroup tg;//= new TransformGroup();
        //   tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

        //    tg = new TransformGroup(t3d);

        for (int ii = 0; ii < i; ii++) {
            for (int jj = 0; jj < j; jj++) {
                for (int kk = 0; kk < k; kk++) {
                    t3d.set(new Vector3f(ii, jj, kk));
                    // tg.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
                    tg = new TransformGroup(t3d);
                    // tg.setTransform(t3d);
                    tg.addChild(getSphere3D(blueApp, r));   // set its radius and appearance
                    sceneBG.addChild(tg);
                }
            }
        }
    }  // end of populateScene()

    private void populateCylinders(int i, int j, int k, float r) {
        Appearance blueApp = getAppearance();

        // position the sphere
        Transform3D t3d;
        t3d = new Transform3D();
        TransformGroup tg;//= new TransformGroup();
        //   tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

        //    tg = new TransformGroup(t3d);

        for (int ii = 0; ii < i; ii++) {
            for (int jj = 0; jj < j; jj++) {
                for (int kk = 0; kk < k; kk++) {
                    t3d.set(new Vector3f(ii, jj, kk));
                    // tg.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
                    tg = new TransformGroup(t3d);
                    // tg.setTransform(t3d);
                    tg.addChild(getCylinder3D(r, 50 * r, blueApp));   // set its radius and appearance
                    sceneBG.addChild(tg);
                }
            }
        }
    }  // end of populateScene()

    private void populateEyeFields(int i, int j, float r, float h) {
        Appearance blueApp = getAppearance();

// position the sphere
        Transform3D t3d;
        t3d = new Transform3D();
        TransformGroup tg;//= new TransformGroup();
//   tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

//    tg = new TransformGroup(t3d);
       double x, y;
        double expIncr = Math.PI * 2 / (double) i;
        double minRadius = 1;
        double theta = 0;
        //  double thetaIncr = Math.PI*2/(double)i;
        for (int ii = 0; ii < i; ii++) {
            double rho = minRadius * Math.exp((double) ii * expIncr);
            theta = 0;
            for (int jj = 0; jj < j; jj++) {

                theta += expIncr;
                x = rho* Math.cos(theta);
                y = rho* Math.sin(theta);
                t3d.set(new Vector3f((float)x, 0, (float)y));
                // tg.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
                tg = new TransformGroup(t3d);
                // tg.setTransform(t3d);
                tg.addChild(getCylinder3D((float)(r*rho*expIncr*.5f), h, blueApp));   // set its radius and appearance
                sceneBG.addChild(tg);

            }
        }
    }  // end of populateScene()

    //private Sphere getSphere3D(Appearance blueApp) {
    private Sphere getSphere3D(Appearance app, float r) {

        Sphere sphere = new Sphere(r, app);
        return sphere;
    }

    private Cylinder getCylinder3D(float radius, float height, Appearance app) {
        Cylinder cylinder;
        cylinder = new Cylinder(radius, height, app);
        return cylinder;
    }

    private Appearance getAppearance() {
        Material blueMat = getMaterial();

        // sets ambient, emissive, diffuse, specular, shininess
        blueMat.setLightingEnable(true);

        Appearance blueApp = new Appearance();
        blueApp.setMaterial(blueMat);
        return blueApp;
    }

    private Material getMaterial() {
        // Create the blue appearance node
        Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
        Color3f blue = new Color3f(0.8f, 0.3f, 0.5f);
        Color3f specular = new Color3f(0.9f, 0.9f, 0.9f);

        Material blueMat = new Material(blue, black, blue, specular, 25.0f);
        return blueMat;
    }

} // end of WrapCheckers3D class